#!/usr/bin/python
# Copyright Paul Giannaros <ceruleanblaze@gmail.com>

import time
import commands
import subprocess

def sayText(text):
    """ Speak some text via DCOP synchronously. """
    # Use subprocess.Popen to prevent any funkiness with shell meta characters.
    job = subprocess.Popen(["dcop", "kttsd", "KSpeech", "setText", "%s" % text.replace("'", "\\'"), "0"], stdout=subprocess.PIPE).communicate()[0]
    subprocess.Popen(["dcop", "kttsd", "KSpeech", "startText", job], stdout=None, stderr=None)
    time.sleep(0.5)
    # Wait till text speaking is done..
    while commands.getoutput("dcop kttsd KSpeech isSpeakingText") == "true":
        time.sleep(0.5)

def pauseresumeAmarok():
    """ Pause/Resume playing. """
    subprocess.Popen(["dcop", "amarok", "player", "pause"])

def configAsDict():
    """ Returns the configuration file as a dictionary of key->value """
    f = file("ttsrc")
    text = f.read().strip()
    f.close()
    return dict([line.split("=", 1) for line in text.splitlines() if line and not line.startswith("#")])

def writeConfigFromDict(d):
    """ Write the configuration file given a dict of keys->value """
    f = file("ttsrc", "w")
    output = ""
    for keyvalue in d.items():
        output += "%s=%s\n" % keyvalue
    f.write(output)
    f.close()

def loadSpeakingString():
    """ Get the string to be spoken, as set by the user (or the default) """
    return configAsDict()["Text"] 

def speakableTrack():
    """ Return the text to speek from the config file using SSML for some breaks """
    string = loadSpeakingString()
    string = string.replace("%a", commands.getoutput("dcop amarok player artist")).replace("%t", commands.getoutput("dcop amarok player title")).replace("%l", commands.getoutput("dcop amarok player totalTime")).replace("%b", commands.getoutput("dcop amarok player album"))
    return """<?xml version="1.0"?><speak><p>%s</p></speak>""" % string
#     commands.getoutput("dcop amarok player title"), commands.getoutput("dcop amarok player artist")

def configure():
    """ Show a configure dialog that allows the user to customize
    the spoken text. """
    config = configAsDict()
    text = subprocess.Popen(["kdialog", "--title", "amaroK tts script - configuration", "--inputbox", "Text to be spoken upon track change.\nCan include any of the following metacharacters:\n\n%a - artist\n%t - title\n%b - album\n%l - length", "%s" % config["Text"]], stdout=subprocess.PIPE).communicate()[0]
    # If cancel was not pressed update the conf.
    print text
    if not text == "\n":
        config["Text"] = text
    if subprocess.call(["kdialog", "--title", "amaroK tts script - configuration", "--yesno", "Shall the song be paused while the announcing is taking place?"]) == 0:
        config["Pause"] = "1"
    else:
        config["Pause"] = "0"
    writeConfigFromDict(config)

def main():
    # Start KTTSD if it's not running
    if not "kttsd" in [proc[proc.rfind(" ") + 1:] for proc in commands.getoutput("ps -A").split("\n")]:
        subprocess.Popen("kttsd")
    # Make sure the config file exists
    try:
        f = file("ttsrc")
        if not f.read():
            raise IOError
        f.close()
    except IOError:
        f = file("ttsrc", "w")
        f.write(r"Text=%t by %a\nPause=0")
        f.close()
    # Event loop
    while True:
        command = raw_input()
        if command == "configure":
            configure()
        if command == "trackChange":
            pause = False
            if configAsDict()["Pause"] == "1":
                pause = True
            # If we should, pause amarok. Speak the song name
            if pause:
                pauseresumeAmarok()
            sayText(speakableTrack())
            if pause:
                pauseresumeAmarok()

if __name__ == "__main__":
    main()
